package de.hdm_stuttgart.mi.sd1.task1;

import java.util.Arrays;

/**
 * Creating arrays from integer range definitions.
 */
public class _3_Range {

    /**
     *
     * <p>Turn a range defined by start and end value into a sorted array of values.</p>
     *
     * <p>A range of unique ascending values is being defined by a lower and upper inclusive limit:</p>
     *
     * <table class="goikTableDefaults">
     *     <caption>Range definitions and resulting arrays.</caption>
     *     <tbody>
     *         <tr>
     *             <th>Range</th>
     *             <th>Array</th>
     *             <th>Remark</th>
     *         </tr>
     *         <tr>
     *             <td>[2, 7]</td>
     *             <td>{2, 3, 4, 5, 6, 7}</td>
     *             <td>All values in between 2 and 7</td>
     *         </tr>
     *         <tr>
     *             <td>[-5, -5]</td>
     *             <td>{-5}</td>
     *             <td>Just the single value -5</td>
     *         </tr>
     *         <tr>
     *             <td>[2, 1]</td>
     *             <td>{}</td>
     *             <td>Range end definition <code>1</code> is smaller than range start <code>2</code> ⇒ empty result.</td>
     *         </tr>
     *
     *     </tbody>
     *
     * </table>
     *
     * @param rangeStartInclusive Start of range (inclusive).
     * @param rangeEndInclusive End of range (inclusive).
     * @return All values within <code>[rangeStartInclusive, rangeEndInclusive]</code>. If
     * <code>rangeEndInclusive &lt; rangeStartInclusive</code> the result is an empty array.
     */
    static public int[] createRange(final int rangeStartInclusive, final int rangeEndInclusive) {
        if (rangeStartInclusive <= rangeEndInclusive) {
            final int size = rangeEndInclusive - rangeStartInclusive + 1;
            final int[] result = new int[size];
            for (int i = 0; i < size; i++) {
                result[i] = rangeStartInclusive + i;
            }
            return result;
        } else {
            return new int[]{};
        }
    }

    /**
     * <p>An array by range definition restricted by an array of values to be excluded.</p>
     *
     * <p>This method generalizes {@link #createRange(int, int)} by excluding values being present
     * in an array. Examples:</p>
     *
     * <table class="goikTableDefaults">
     *     <caption>Range definitions restricted by exclusions.</caption>
     *     <tbody>
     *         <tr>
     *             <th>[rangeStartInclusive, rangeEndInclusive]</th>
     *             <th>Excluded Values</th>
     *             <th>Result array</th>
     *             <th>Explanation</th>
     *         </tr>
     *         <tr>
     *             <td>[2, 7]</td>
     *             <td>{1, 4, 6}</td>
     *             <td>{2, 3, 5, 7}</td>
     *             <td>\( \{2, 3, \color{red}{4}, 5, \color{red}{6}, 7\} \) by range excluding
     *             \( \{\color{red}{1}, \color{red}{4}, \color{red}{6}\} \Longrightarrow \{2, 3, 5, 7\}\)  </td>
     *         </tr>
     *         <tr>
     *             <td>[-1, 3]</td>
     *             <td>{-1, 1, 2, 3}</td>
     *             <td>{0}</td>
     *             <td>\( \{\color{red}{-1}, 0, \color{red}{1}, \color{red}{2}, \color{red}{3}\} \) by range excluding
     *             \( \{\color{red}{-1}, \color{red}{1}, \color{red}{2}, \color{red}{3} \} \)
     *             \(\Longrightarrow \{0\}\) </td>
     *         </tr>
     *         <tr>
     *             <td>[1, 3]</td>
     *             <td>\( \{2, 1, 3, 4 \} \)</td>
     *             <td>{}</td>
     *             <td>\( \{\color{red}{1}, \color{red}{2}, \color{red}{3}\} \) by range excluding
     *             \( \{ \color{red}{2}, \color{red}{1}, \color{red}{3}, \color{red}{4} \} \)
     *             \(\Longrightarrow \{\}\)  </td>
     *         </tr>
     *
     *     </tbody>
     *
     * </table>
     *
     * <section class="implementationHints">
     *    <h4 class="implementationHints">Implementation hints:</h4>
     *    <p>The class {@link Arrays} offers assistance i.e. for copying arrays.</p>
     * </section>
     *
     * @param rangeStartInclusive Values greater than or equal
     * @param rangeEndInclusive Start of range (inclusive).
     * @param excludedValues End of range (inclusive).
     * @return All values within <code>[rangeStartInclusive, rangeEndInclusive]</code> <em>not</em> being
     * present in the <code>excludedValues</code> array.
     */
    static public int[]
    createRangeWithExclusions(final int rangeStartInclusive, final int rangeEndInclusive, final int[] excludedValues) {
        //                                    2                          5    [1,2,3,4,5]             [2,10]
        if (rangeStartInclusive <= rangeEndInclusive) {
            final int[] tmpResult = new int[rangeEndInclusive - rangeStartInclusive + 1]; // [1,3,4,5,0]
            int resultValueCount = 0;
            for (int i = rangeStartInclusive; i <= rangeEndInclusive; i++) {
//              if (!contains(excludedValues, i)) {
                if (!Arrays.stream(excludedValues).anyMatch(  // without contains(...)
                    Integer.valueOf(i)::equals)) {            // method below.
                    tmpResult[resultValueCount++] = i;
                }
            }
            return Arrays.copyOf(tmpResult, resultValueCount);
        } else {
            return new int[]{};
        }
    }

    /**
     * Check for presence of a value within an array.
     *
     * @param list An array of values
     * @param value To be checked for being contained within the array in question.
     * @return
     */
    static private boolean contains(final int[] list, final int value) {
        for (final int l: list) {
            if (l == value) {
                return true;
            }
        }
        return false;
    }

    private _3_Range(){/* Ignore me: Suppressing default constructor javadoc generation:
                                    javadoc does not include private constructors by default */}
}
