package de.hdm_stuttgart.mi.sd1.task2;

/**
 * <p>This class converts a given number of days into an immutable object providing years, weeks and days.</p>
 *
 * <p>The following example decomposes a 380 days period into an equivalent number of years, weeks and days, each to
 * its minimum. We assume all years having 365 days ignoring leap years:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Decomposing a days period into a minimal number of years, weeks and days.</caption>
 *     <tbody>
 *         <tr>
 *             <th>Code</th>
 *             <th>Result</th>
 *         </tr>
 *         <tr>
 *             <td>
 *                 <pre><code class="java">  final _5_DayConverter converter = new _5_DayConverter(380); // 380 days
 *
 *  final int
 *      years = converter.years,
 *      weeks = converter.weeks,
 *      days  = converter.days;
 *
 *  System.out.println(380 + " days equal " + years + " year, " + weeks + " weeks and " + days + " day");</code></pre>
 *             </td>
 *             <td>
 *                 <pre><code class="nohighlight">380 days equal 1 year, 2 weeks and 1 day</code></pre>
 *             </td>
 *         </tr>
 *     </tbody>
 *
 * </table>
 *
 * <p>Instances of {@link _5_DayConverter} are immutable. Thus the following assignments are bound to fail
 * at compile time:</p>
 *
 * <pre><code class="java"> converter.years = 2;
 * converter.weeks = 1;
 * converter.days = 5;</code></pre>
 *
 */
public class _5_DayConverter {

  static private final int
          DAYS_PER_YEAR = 365,
          DAYS_PER_WEEK = 7;

  /**
   * A given time periods number of years.
   */
  public final int years;

  /**
   * A given time periods number of weeks.
   */
  public final int weeks;

  /**
   * A given time periods number of days.
   */
  public final int days;

  /**
   * Creating an instance by a given day period.
   *
   * @param days A number of days possibly exceeding 365.
   */
  public _5_DayConverter(final int days) {
    years = days / DAYS_PER_YEAR;
    final int remainingDays = days % DAYS_PER_YEAR;
    weeks = remainingDays / DAYS_PER_WEEK;
    this.days = remainingDays % DAYS_PER_WEEK;
  }
}
